@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -16,10 +16,9 @@
 
 #include <string>
 
-namespace grk {
-
+namespace grk
+{
 // transfer metadata
 void transferExifTags(std::string src, std::string dest);
 
-
-}
+} // namespace grk
