@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -20,19 +20,21 @@
 
 #include "IFileIO.h"
 
-class FileStreamIO : public IFileIO {
-public:
+class FileStreamIO : public IFileIO
+{
+  public:
 	FileStreamIO();
 	virtual ~FileStreamIO() override;
 	bool open(std::string fileName, std::string mode) override;
 	bool close(void) override;
-	bool write(uint8_t *buf, size_t len) override;
-	bool read(uint8_t *buf, size_t len) override;
-	bool seek(int64_t pos) override;
-	FILE* getFileStream(){ return m_fileHandle;}
+	uint64_t write(uint8_t* buf, uint64_t offset, size_t len, size_t maxLen, bool pooled) override;
+	uint64_t write(GrkSerializeBuf buffer) override;
+	bool read(uint8_t* buf, size_t len) override;
+	uint64_t seek(int64_t off, int whence) override;
+	FILE* getFileStream(void);
+	int getFileDescriptor(void);
 
-private:
-	FILE *m_fileHandle;
-	std::string m_fileName;
+  private:
+	FILE* fileHandle_;
+	std::string fileName_;
 };
-
