@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -23,28 +23,28 @@
 #include <cstring>
 
 #ifndef GROK_HAVE_LIBJPEG
-# error GROK_HAVE_LIBJPEG_NOT_DEFINED
+#error GROK_HAVE_LIBJPEG_NOT_DEFINED
 #endif /* GROK_HAVE_LIBJPEG */
 
-
 #include <setjmp.h>
 #include <cassert>
 
 #include "common.h"
 #include "FileStreamIO.h"
 
-
-struct my_error_mgr {
+struct my_error_mgr
+{
 	struct jpeg_error_mgr pub; /* "public" fields */
 
 	jmp_buf setjmp_buffer; /* for return to caller */
 };
 
-typedef struct my_error_mgr *my_error_ptr;
+typedef struct my_error_mgr* my_error_ptr;
 
-METHODDEF(void) my_error_exit(j_common_ptr cinfo) {
+METHODDEF(void) my_error_exit(j_common_ptr cinfo)
+{
 	/* cinfo->err really points to a my_error_mgr struct, so coerce pointer */
-	my_error_ptr myerr = (my_error_ptr) cinfo->err;
+	my_error_ptr myerr = (my_error_ptr)cinfo->err;
 
 	/* Always display the message. */
 	/* We could postpone this until after returning, if we chose. */
@@ -79,20 +79,19 @@ METHODDEF(void) my_error_exit(j_common_ptr cinfo) {
  * temporary files are deleted if the program is interrupted.  See libjpeg.txt.
  */
 
+grk_image* JPEGFormat::jpegtoimage(const char* filename, grk_cparameters* parameters)
+{
+	readFromStdin = grk::useStdio(std::string(filename));
 
-grk_image* JPEGFormat::jpegtoimage(const char *filename,
-		grk_cparameters *parameters) {
-	readFromStdin = grk::useStdio(filename);
-
-	int32_t *planes[3];
+	int32_t* planes[3];
 	JDIMENSION w = 0, h = 0;
 	uint32_t dest_stride;
-	int bps = 0, numcomps = 0;
+	int bps = 0, decompressNumComps = 0;
 	cvtTo32 cvtJpegTo32s;
 	GRK_COLOR_SPACE color_space = GRK_CLRSPC_UNKNOWN;
-	grk_image_cmptparm cmptparm[3]; /* mono or RGB */
+	grk_image_comp cmptparm[3]; /* mono or RGB */
 	cvtInterleavedToPlanar cvtToPlanar;
-	JOCTET *icc_data_ptr = nullptr;
+	JOCTET* icc_data_ptr = nullptr;
 	unsigned int icc_data_len = 0;
 
 	/* This struct contains the JPEG decompression parameters and pointers to
@@ -114,12 +113,16 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 	 * requires it in order to read binary files.
 	 */
 
-	if (readFromStdin) {
-		if (!grk::grk_set_binary_mode(stdin))
+	if(readFromStdin)
+	{
+		if(!grk::grk_set_binary_mode(stdin))
 			return nullptr;
-		m_fileStream = stdin;
-	} else {
-		if ((m_fileStream = fopen(filename, "rb")) == nullptr) {
+		fileStream_ = stdin;
+	}
+	else
+	{
+		if((fileStream_ = fopen(filename, "rb")) == nullptr)
+		{
 			spdlog::error("can't open {}", filename);
 			return 0;
 		}
@@ -131,7 +134,8 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 	cinfo.err = jpeg_std_error(&jerr.pub);
 	jerr.pub.error_exit = my_error_exit;
 	/* Establish the setjmp return context for my_error_exit to use. */
-	if (setjmp(jerr.setjmp_buffer)) {
+	if(setjmp(jerr.setjmp_buffer))
+	{
 		success = false;
 		goto cleanup;
 	}
@@ -140,11 +144,11 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 	setup_read_icc_profile(&cinfo);
 
 	/* Step 2: specify data source (eg, a file) */
-	jpeg_stdio_src(&cinfo, m_fileStream);
+	jpeg_stdio_src(&cinfo, fileStream_);
 
 	/* Step 3: read file parameters with jpeg_read_header() */
 
-	(void) jpeg_read_header(&cinfo, TRUE);
+	GRK_UNUSED(jpeg_read_header)(&cinfo, TRUE);
 	/* We can ignore the return value from jpeg_read_header since
 	 *   (a) suspension is not possible with the stdio data source, and
 	 *   (b) we passed TRUE to reject a tables-only JPEG file as an error.
@@ -152,7 +156,8 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 	 */
 
 	// read ICC profile
-	if (!read_icc_profile(&cinfo, &icc_data_ptr, &icc_data_len)) {
+	if(!read_icc_profile(&cinfo, &icc_data_ptr, &icc_data_len))
+	{
 		spdlog::warn("jpegtoimage: Failed to read ICC profile");
 	}
 
@@ -164,31 +169,33 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 
 	/* Step 5: Start decompressor */
 
-	(void) jpeg_start_decompress(&cinfo);
+	GRK_UNUSED(jpeg_start_decompress)(&cinfo);
 	/* We can ignore the return value since suspension is not possible
 	 * with the stdio data source.
 	 */
 
 	bps = cinfo.data_precision;
-	if (bps != 8) {
-		spdlog::error("jpegtoimage: Unsupported m_image precision {}", bps);
+	if(bps != 8)
+	{
+		spdlog::error("jpegtoimage: Unsupported image_ precision {}", bps);
 		success = false;
 		goto cleanup;
 	}
 
-	numcomps = cinfo.output_components;
+	decompressNumComps = cinfo.output_components;
 	w = cinfo.image_width;
 	h = cinfo.image_height;
 	cvtJpegTo32s = cvtTo32_LUT[bps];
-	memset(&cmptparm[0], 0, 3 * sizeof(grk_image_cmptparm));
-	cvtToPlanar = cvtInterleavedToPlanar_LUT[numcomps];
+	memset(&cmptparm[0], 0, 3 * sizeof(grk_image_comp));
+	cvtToPlanar = cvtInterleavedToPlanar_LUT[decompressNumComps];
 
-	if (cinfo.output_components == 3)
+	if(cinfo.output_components == 3)
 		color_space = GRK_CLRSPC_SRGB;
 	else
 		color_space = GRK_CLRSPC_GRAY;
 
-	for (int j = 0; j < cinfo.output_components; j++) {
+	for(int j = 0; j < cinfo.output_components; j++)
+	{
 		cmptparm[j].prec = (uint8_t)bps;
 		cmptparm[j].dx = 1;
 		cmptparm[j].dy = 1;
@@ -196,64 +203,63 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 		cmptparm[j].h = h;
 	}
 
-	m_image = grk_image_create((uint16_t)numcomps, &cmptparm[0], color_space,true);
-	if (!m_image) {
+	image_ = grk_image_new((uint16_t)decompressNumComps, &cmptparm[0], color_space);
+	if(!image_)
+	{
 		success = false;
 		goto cleanup;
 	}
-	if (icc_data_ptr && icc_data_len) {
-		m_image->color.icc_profile_buf = new uint8_t[icc_data_len];
-		memcpy(m_image->color.icc_profile_buf, icc_data_ptr, icc_data_len);
-		m_image->color.icc_profile_len = icc_data_len;
+	if(icc_data_ptr && icc_data_len)
+	{
+		copy_icc(image_, icc_data_ptr, icc_data_len);
 		icc_data_len = 0;
 	}
-	if (icc_data_ptr)
-		free(icc_data_ptr);
+	free(icc_data_ptr);
 	icc_data_ptr = nullptr;
-	/* set m_image offset and reference grid */
-	m_image->x0 = parameters->image_offset_x0;
-	m_image->x1 =
-			!m_image->x0 ?
-					(w - 1) * 1 + 1 : m_image->x0 + (w - 1) * 1 + 1;
-	if (m_image->x1 <= m_image->x0) {
-		spdlog::error("jpegtoimage: Bad value for m_image->x1({}) vs. "
-				"m_image->x0({}).", m_image->x1,
-				m_image->x0);
+	/* set image_ offset and reference grid */
+	image_->x0 = parameters->image_offset_x0;
+	image_->x1 = !image_->x0 ? (w - 1) * 1 + 1 : image_->x0 + (w - 1) * 1 + 1;
+	if(image_->x1 <= image_->x0)
+	{
+		spdlog::error("jpegtoimage: Bad value for image_->x1({}) vs. "
+					  "image_->x0({}).",
+					  image_->x1, image_->x0);
 		success = false;
 		goto cleanup;
 	}
 
-	m_image->y0 = parameters->image_offset_y0;
-	m_image->y1 =
-			!m_image->y0 ?
-					(h - 1) * 1 + 1 : m_image->y0 + (h - 1) * 1 + 1;
+	image_->y0 = parameters->image_offset_y0;
+	image_->y1 = !image_->y0 ? (h - 1) * 1 + 1 : image_->y0 + (h - 1) * 1 + 1;
 
-	if (m_image->y1 <= m_image->y0) {
-		spdlog::error("jpegtoimage: Bad value for m_image->y1({}) vs. "
-				"m_image->y0({}).", m_image->y1,
-				m_image->y0);
+	if(image_->y1 <= image_->y0)
+	{
+		spdlog::error("jpegtoimage: Bad value for image_->y1({}) vs. "
+					  "image_->y0({}).",
+					  image_->y1, image_->y0);
 		success = false;
 		goto cleanup;
 	}
 
-	for (int j = 0; j < numcomps; j++) {
-		planes[j] = m_image->comps[j].data;
+	for(int j = 0; j < decompressNumComps; j++)
+	{
+		planes[j] = image_->comps[j].data;
 	}
 
-	buffer32s = new int32_t[w * (size_t)numcomps];
+	buffer32s = new int32_t[w * (size_t)decompressNumComps];
 
 	/* We may need to do some setup of our own at this point before reading
 	 * the data.  After jpeg_start_decompress() we have the correct scaled
-	 * output m_image dimensions available, as well as the output colormap
+	 * output image_ dimensions available, as well as the output colormap
 	 * if we asked for color quantization.
 	 * In this example, we need to make an output work buffer of the right size.
 	 */
 	/* JSAMPLEs per row in output buffer */
 	row_stride = (int)cinfo.output_width * cinfo.output_components;
-	/* Make a one-row-high sample array that will go away when done with m_image */
-	buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo, JPOOL_IMAGE,
-			(JDIMENSION)row_stride, 1);
-	if (!buffer) {
+	/* Make a one-row-high sample array that will go away when done with image_ */
+	buffer =
+		(*cinfo.mem->alloc_sarray)((j_common_ptr)&cinfo, JPOOL_IMAGE, (JDIMENSION)row_stride, 1);
+	if(!buffer)
+	{
 		success = false;
 		goto cleanup;
 	}
@@ -264,20 +270,20 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 	/* Here we use the library's state variable cinfo.output_scanline as the
 	 * loop counter, so that we don't have to keep track ourselves.
 	 */
-	dest_stride = m_image->comps[0].stride;
-	while (cinfo.output_scanline < cinfo.output_height) {
+	dest_stride = image_->comps[0].stride;
+	while(cinfo.output_scanline < cinfo.output_height)
+	{
 		/* jpeg_read_scanlines expects an array of pointers to scanlines.
 		 * Here the array is only one element long, but you could ask for
 		 * more than one scanline at a time if that's more convenient.
 		 */
-		(void) jpeg_read_scanlines(&cinfo, buffer, 1);
+		GRK_UNUSED(jpeg_read_scanlines)(&cinfo, buffer, 1);
 
 		// convert 8 bit buffer to 32 bit buffer
-		cvtJpegTo32s(buffer[0], buffer32s, (size_t) w * (size_t) numcomps,
-				false);
+		cvtJpegTo32s(buffer[0], buffer32s, (size_t)w * (size_t)decompressNumComps, false);
 
 		// convert to planar
-		cvtToPlanar(buffer32s, planes, (size_t) w);
+		cvtToPlanar(buffer32s, planes, (size_t)w);
 
 		planes[0] += dest_stride;
 		planes[1] += dest_stride;
@@ -286,12 +292,12 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 
 	/* Step 7: Finish decompression */
 
-	(void) jpeg_finish_decompress(&cinfo);
-	/* We can ignore the return value since suspension is not possible
-	 * with the stdio data source.
-	 */
-	cleanup: if (icc_data_ptr)
-		free(icc_data_ptr);
+	GRK_UNUSED(jpeg_finish_decompress)(&cinfo);
+/* We can ignore the return value since suspension is not possible
+ * with the stdio data source.
+ */
+cleanup:
+	free(icc_data_ptr);
 	jpeg_destroy_decompress(&cinfo);
 
 	delete[] buffer32s;
@@ -299,51 +305,53 @@ grk_image* JPEGFormat::jpegtoimage(const char *filename,
 	/* At this point you may want to check to see whether any corrupt-data
 	 * warnings occurred (test whether jerr.pub.num_warnings is nonzero).
 	 */
-	if (jerr.pub.num_warnings != 0) {
+	if(jerr.pub.num_warnings != 0)
+	{
 		spdlog::warn("JPEG library reported {} of corrupt data warnings",
-					(int) jerr.pub.num_warnings);
+					 (int)jerr.pub.num_warnings);
 	}
 
-	if (!success) {
-		grk_image_destroy(m_image);
-		m_image = nullptr;
+	if(!success)
+	{
+		grk_object_unref(&image_->obj);
+		image_ = nullptr;
 	}
 	/* After finish_decompress, we can close the input file.
 	 * Here we postpone it until after no more JPEG errors are possible,
 	 * so as to simplify the setjmp error logic above.  (Actually, I don't
 	 * think that jpeg_destroy can do an error exit, but why assume anything...)
 	 */
-	if (m_fileStream && !readFromStdin) {
-		if (!grk::safe_fclose(m_fileStream)) {
-			grk_image_destroy(m_image);
-			m_image = nullptr;
+	if(fileStream_ && !readFromStdin)
+	{
+		if(!grk::safe_fclose(fileStream_))
+		{
+			grk_object_unref(&image_->obj);
+			image_ = nullptr;
 		}
 	}
-	return m_image;
-}/* jpegtoimage() */
+	return image_;
+} /* jpegtoimage() */
 
-JPEGFormat::JPEGFormat(void) : success(true), buffer(nullptr), buffer32s(
-		nullptr), color_space(JCS_UNKNOWN), adjust(0),readFromStdin(false), planes{0,0,0},
-		cvtPxToCx(nullptr), 	cvtTo8bpp(nullptr)
+JPEGFormat::JPEGFormat(void)
+	: success(true), buffer(nullptr), buffer32s(nullptr), color_space(JCS_UNKNOWN), adjust(0),
+	  readFromStdin(false), planes{0, 0, 0}
+{}
+
+bool JPEGFormat::encodeHeader(void)
 {
+	if(isHeaderEncoded())
+		return true;
 
-}
-
-bool JPEGFormat::encodeHeader(grk_image *image, const std::string &filename,
-		uint32_t compressionParam) {
-
-	if (!image)
-		return false;
-	m_image = image;
 	int32_t firstAlpha = -1;
 	size_t numAlphaChannels = 0;
-	uint32_t numcomps = m_image->numcomps;
-	uint32_t sgnd = m_image->comps[0].sgnd;
-	adjust = sgnd ? 1 << (m_image->comps[0].prec - 1) : 0;
-	uint32_t width = m_image->comps[0].w;
+	uint32_t width = image_->decompressWidth;
 
 	// actual bits per sample
-	uint8_t prec = m_image->comps[0].prec;
+	uint8_t prec = image_->decompressPrec;
+	uint16_t decompressNumComps = image_->decompressNumComps;
+	uint32_t sgnd = image_->comps[0].sgnd;
+	adjust = sgnd ? 1 << (prec - 1) : 0;
+
 	uint32_t i = 0;
 
 	struct my_error_mgr jerr;
@@ -356,7 +364,6 @@ bool JPEGFormat::encodeHeader(grk_image *image, const std::string &filename,
 	 * address which we place into the link field in cinfo.
 	 */
 
-
 	/* This struct represents a JPEG error handler.  It is declared separately
 	 * because applications often want to supply a specialized error handler
 	 * (see the second half of this file for an example).  But here we just
@@ -365,119 +372,91 @@ bool JPEGFormat::encodeHeader(grk_image *image, const std::string &filename,
 	 * Note that this struct must live as long as the main JPEG parameter
 	 * struct, to avoid dangling-pointer problems.
 	 */
-	JDIMENSION image_width = m_image->x1 - m_image->x0; /* input m_image width */
-	JDIMENSION image_height = m_image->y1 - m_image->y0; /* input m_image height */
+	JDIMENSION image_width = image_->x1 - image_->x0; /* input image_ width */
+	JDIMENSION image_height = image_->y1 - image_->y0; /* input image_ height */
 
-	switch (m_image->color_space) {
-	case GRK_CLRSPC_SRGB: /**< sRGB */
-		color_space = JCS_RGB;
-		break;
-	case GRK_CLRSPC_GRAY: /**< grayscale */
-		color_space = JCS_GRAYSCALE;
-		break;
-	case GRK_CLRSPC_SYCC: /**< YUV */
-		color_space = JCS_YCbCr;
-		break;
-	case GRK_CLRSPC_EYCC: /**< e-YCC */
-		color_space = JCS_YCCK;
-		break;
-	case GRK_CLRSPC_CMYK: /**< CMYK */
-		color_space = JCS_CMYK;
-		break;
-	default:
-		if (numcomps == 3)
+	// sub-sampling not supported at the moment
+	if(isFinalOutputSubsampled(image_))
+	{
+		spdlog::error("JPEGFormat::encodeHeader: subsampling not currently supported.");
+		return false;
+	}
+
+	switch(image_->color_space)
+	{
+		case GRK_CLRSPC_SRGB: /**< sRGB */
 			color_space = JCS_RGB;
-		else if (numcomps == 1)
+			break;
+		case GRK_CLRSPC_GRAY: /**< grayscale */
 			color_space = JCS_GRAYSCALE;
-		else {
-			spdlog::error("imagetojpeg: colour space must be "
-					"either RGB or Grayscale");
-			return false;
-		}
-		break;
+			break;
+		case GRK_CLRSPC_SYCC: /**< YUV */
+			color_space = JCS_YCbCr;
+			break;
+		case GRK_CLRSPC_EYCC: /**< e-YCC */
+			color_space = JCS_YCCK;
+			break;
+		case GRK_CLRSPC_CMYK: /**< CMYK */
+			color_space = JCS_CMYK;
+			break;
+		default:
+			if(decompressNumComps == 3)
+				color_space = JCS_RGB;
+			else if(decompressNumComps == 1)
+				color_space = JCS_GRAYSCALE;
+			else
+			{
+				spdlog::error("JPEGFormat::encodeHeader: unrecognized colour space");
+			}
+			break;
 	}
 
-	if (m_image->numcomps > 4) {
-		spdlog::error("imagetojpeg: number of components {} "
-				"is greater than 4.", m_image->numcomps);
+	if(image_->decompressNumComps > 4)
+	{
+		spdlog::error("JPEGFormat::encodeHeader: number of components {} "
+					  "is greater than 4.",
+					  image_->decompressNumComps);
 		return false;
 	}
+	if(!allComponentsSanityCheck(image_, true))
+		return false;
 
-	planes[0] = m_image->comps[0].data;
-	if (prec == 0) {
-		spdlog::error("imagetojpeg: m_image precision is zero.");
+	planes[0] = image_->comps[0].data;
+	for(i = 1U; i < decompressNumComps; ++i)
+		planes[i] = image_->comps[i].data;
+
+	if(prec != 1 && prec != 2 && prec != 4 && prec != 8)
+	{
+		spdlog::error("JPEGFormat::encodeHeader: can not create {}\n\twrong bit_depth {}",
+					  fileName_, prec);
 		return false;
 	}
-
-	//check for null m_image components
-	for (uint32_t i = 0; i < numcomps; ++i) {
-		auto comp = m_image->comps[i];
-		if (!comp.data) {
-			spdlog::error("imagetojpeg: component {} is null.", i);
-			return false;
-		}
-	}
-	for (i = 1U; i < numcomps; ++i) {
-		if (m_image->comps[0].dx != m_image->comps[i].dx)
-			break;
-		if (m_image->comps[0].dy != m_image->comps[i].dy)
-			break;
-		if (m_image->comps[0].prec != m_image->comps[i].prec)
-			break;
-		if (m_image->comps[0].sgnd != m_image->comps[i].sgnd)
-			break;
-		planes[i] = m_image->comps[i].data;
-	}
-	if (i != numcomps) {
-		spdlog::error(
-				"imagetojpeg: All components shall have the same subsampling, same bit depth.");
-		return false;
-	}
-
-	if (prec < 8 && numcomps > 1) { /* GRAY_ALPHA, RGB, RGB_ALPHA */
-		for (i = 0; i < numcomps; ++i)
-			scale_component(&(m_image->comps[i]), 8);
-		prec = 8;
-	} else if ((prec > 1) && (prec < 8) && ((prec == 6) || ((prec & 1) == 1))) { /* GRAY with non native precision */
-		if ((prec == 5) || (prec == 6))
-			prec = 8;
-		else
-			prec++;
-		for (i = 0; i < numcomps; ++i)
-			scale_component(&(m_image->comps[i]), prec);
-	}
-
-	if (prec != 1 && prec != 2 && prec != 4 && prec != 8) {
-		spdlog::error("imagetojpeg: can not create {}\n\twrong bit_depth {}",
-				filename, prec);
-		return false;
-	}
-
-	cvtPxToCx = cvtPlanarToInterleaved_LUT[numcomps];
-	cvtTo8bpp = cvtFrom32_LUT[prec];
-
 	// Alpha channels
-	for (i = 0U; i < numcomps; ++i) {
-		if (m_image->comps[i].type) {
-			if (firstAlpha == -1)
+	for(i = 0U; i < decompressNumComps; ++i)
+	{
+		if(image_->comps[i].type)
+		{
+			if(firstAlpha == -1)
 				firstAlpha = 0;
 			numAlphaChannels++;
 		}
 	}
-	// We assume that alpha channels occur as last channels in m_image.
-	if (numAlphaChannels && ( (uint32_t)firstAlpha + numAlphaChannels >= numcomps)) {
-		spdlog::warn("PNG requires that alpha channels occur"
-				" as last channels in m_image.");
+	// We assume that alpha channels occur as last channels in image_.
+	if(numAlphaChannels && ((uint32_t)firstAlpha + numAlphaChannels >= decompressNumComps))
+	{
+		spdlog::warn("JPEGFormat::encodeHeader: PNG requires that alpha channels occur"
+					 " as last channels in image_.");
 		numAlphaChannels = 0;
 	}
-	buffer = new uint8_t[width * numcomps];
-	buffer32s = new int32_t[width * numcomps];
+	buffer = new uint8_t[width * decompressNumComps];
+	buffer32s = new int32_t[width * decompressNumComps];
 
 	/* We set up the normal JPEG error routines, then override error_exit. */
 	cinfo.err = jpeg_std_error(&jerr.pub);
 	jerr.pub.error_exit = my_error_exit;
 	/* Establish the setjmp return context for my_error_exit to use. */
-	if (setjmp(jerr.setjmp_buffer)) {
+	if(setjmp(jerr.setjmp_buffer))
+	{
 		/* If we get here, the JPEG code has signaled an error.
 		 * We need to clean up the JPEG object, close the input file, and return.
 		 */
@@ -495,21 +474,20 @@ bool JPEGFormat::encodeHeader(grk_image *image, const std::string &filename,
 	 * VERY IMPORTANT: use "b" option to fopen() if you are on a machine that
 	 * requires it in order to write binary files.
 	 */
-	if (!ImageFormat::encodeHeader(m_image,filename,compressionParam))
+	if(!openFile())
 		return false;
 
-	m_fileStream = ((FileStreamIO*)m_fileIO)->getFileStream();
-	jpeg_stdio_dest(&cinfo, m_fileStream);
+	jpeg_stdio_dest(&cinfo, fileStream_);
 
 	/* Step 3: set parameters for compression */
 
-	/* First we supply a description of the input m_image.
+	/* First we supply a description of the input image_.
 	 * Four fields of the cinfo struct must be filled in:
 	 */
-	cinfo.image_width = image_width; /* m_image width and height, in pixels */
+	cinfo.image_width = image_width; /* image_ width and height, in pixels */
 	cinfo.image_height = image_height;
-	cinfo.input_components = (int)numcomps; /* # of color components per pixel */
-	cinfo.in_color_space = color_space; /* colorspace of input m_image */
+	cinfo.input_components = (int)decompressNumComps; /* # of color components per pixel */
+	cinfo.in_color_space = color_space; /* colorspace of input image_ */
 
 	/* Now use the library's routine to set default compression parameters.
 	 * (You must set at least cinfo.in_color_space before calling this,
@@ -521,15 +499,17 @@ bool JPEGFormat::encodeHeader(grk_image *image, const std::string &filename,
 	 * Here we just illustrate the use of quality (quantization table) scaling:
 	 */
 	jpeg_set_quality(&cinfo,
-			(int)((compressionParam == GRK_DECOMPRESS_COMPRESSION_LEVEL_DEFAULT) ?
-					90 : compressionParam),
-			(boolean) TRUE /* limit to baseline-JPEG values */);
+					 (int)((compressionLevel_ == GRK_DECOMPRESS_COMPRESSION_LEVEL_DEFAULT)
+							   ? 90
+							   : compressionLevel_),
+					 (boolean)TRUE /* limit to baseline-JPEG values */);
 
 	// set resolution
-	if (m_image->capture_resolution[0] > 0 && m_image->capture_resolution[1] > 0) {
+	if(image_->capture_resolution[0] > 0 && image_->capture_resolution[1] > 0)
+	{
 		cinfo.density_unit = 2; // dots per cm
-		cinfo.X_density = (uint16_t)(m_image->capture_resolution[0]/100.0 + 0.5);
-		cinfo.Y_density = (uint16_t)(m_image->capture_resolution[1]/100.0 + 0.5);
+		cinfo.X_density = (uint16_t)(image_->capture_resolution[0] / 100.0 + 0.5);
+		cinfo.Y_density = (uint16_t)(image_->capture_resolution[1] / 100.0 + 0.5);
 	}
 
 	/* Step 4: Start compressor */
@@ -537,19 +517,18 @@ bool JPEGFormat::encodeHeader(grk_image *image, const std::string &filename,
 	/* TRUE ensures that we will write a complete interchange-JPEG file.
 	 * Pass TRUE unless you are very sure of what you're doing.
 	 */
-	jpeg_start_compress(&cinfo, (boolean) TRUE);
-	if (m_image->color.icc_profile_buf) {
-		write_icc_profile(&cinfo, m_image->color.icc_profile_buf,
-				m_image->color.icc_profile_len);
+	jpeg_start_compress(&cinfo, (boolean)TRUE);
+	if(image_->meta && image_->meta->color.icc_profile_buf)
+	{
+		write_icc_profile(&cinfo, image_->meta->color.icc_profile_buf,
+						  image_->meta->color.icc_profile_len);
 	}
+	encodeState = IMAGE_FORMAT_ENCODED_HEADER;
 
-    return true;
-
-
+	return true;
 }
-bool JPEGFormat::encodeStrip(uint32_t rows){
-	(void)rows;
-
+bool JPEGFormat::encodePixels(void)
+{
 	/* Step 5: while (scan lines remain to be written) */
 	/*           jpeg_write_scanlines(...); */
 
@@ -558,28 +537,28 @@ bool JPEGFormat::encodeStrip(uint32_t rows){
 	 * To keep things simple, we pass one scanline per call; you can pass
 	 * more if you wish, though.
 	 */
-
-	while (cinfo.next_scanline < cinfo.image_height) {
+	auto iter = grk::InterleaverFactory<int32_t>::makeInterleaver(8);
+	if(!iter)
+		return false;
+	while(cinfo.next_scanline < cinfo.image_height)
+	{
 		/* jpeg_write_scanlines expects an array of pointers to scanlines.
 		 * Here the array is only one element long, but you could pass
 		 * more than one scanline at a time if that's more convenient.
 		 */
-		uint32_t stride = m_image->comps[0].stride;
-		cvtPxToCx(planes, buffer32s, (size_t) m_image->comps[0].w, adjust);
-		cvtTo8bpp(buffer32s, (uint8_t*) buffer,
-				(size_t) m_image->comps[0].w * m_image->numcomps);
+		iter->interleave((int32_t**)planes, image_->decompressNumComps, (uint8_t*)buffer,
+						 image_->decompressWidth, image_->comps[0].stride, image_->decompressWidth,
+						 1, adjust);
 		JSAMPROW row_pointer[1]; /* pointer to JSAMPLE row[s] */
 		row_pointer[0] = buffer;
-		planes[0] += stride;
-		planes[1] += stride;
-		planes[2] += stride;
-		(void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
+		GRK_UNUSED(jpeg_write_scanlines)(&cinfo, row_pointer, 1);
 	}
+	delete iter;
 
 	return true;
 }
-bool JPEGFormat::encodeFinish(void){
-
+bool JPEGFormat::encodeFinish(void)
+{
 	/* Step 6: Finish compression */
 	jpeg_finish_compress(&cinfo);
 
@@ -595,8 +574,7 @@ bool JPEGFormat::encodeFinish(void){
 	return ImageFormat::encodeFinish() && success;
 }
 
-grk_image* JPEGFormat::decode(const std::string &filename,
-		grk_cparameters *parameters) {
+grk_image* JPEGFormat::decode(const std::string& filename, grk_cparameters* parameters)
+{
 	return jpegtoimage(filename.c_str(), parameters);
 }
-
