@@ -2,6 +2,9 @@
 
 namespace tf {
 
+// ----------------------------------------------------------------------------
+
+// class: TopologyBase
 class TopologyBase {
   
   friend class Executor;
@@ -12,7 +15,7 @@ class TopologyBase {
 
   protected:
 
-  bool _is_cancelled { false };
+  std::atomic<bool> _is_cancelled { false };
 };
 
 // ----------------------------------------------------------------------------
@@ -27,6 +30,7 @@ class AsyncTopology : public TopologyBase {
 class Topology : public TopologyBase {
   
   friend class Executor;
+  friend class Runtime;
 
   public:
 
@@ -39,7 +43,7 @@ class Topology : public TopologyBase {
 
     std::promise<void> _promise;
 
-    std::vector<Node*> _sources;
+    SmallVector<Node*> _sources;
 
     std::function<bool()> _pred;
     std::function<void()> _call;
