@@ -1,5 +1,5 @@
 /**
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -26,23 +26,21 @@
 #ifdef _WIN32
 typedef HMODULE dynamic_handle_t;
 #else
-typedef void *dynamic_handle_t;
+typedef void* dynamic_handle_t;
 #endif
 
-namespace grk {
-
-struct minpf_dynamic_library {
-
+namespace grk
+{
+struct minpf_dynamic_library
+{
 	char path[MINPF_MAX_PATH_LEN];
 	dynamic_handle_t handle;
-
 };
 
-minpf_dynamic_library* minpf_load_dynamic_library(const char *path,
-		char *error);
-bool minpf_unload_dynamic_library(minpf_dynamic_library *library);
-void* minpf_get_symbol(minpf_dynamic_library *library, const char *symbol);
-bool minpf_get_full_path(const char *path, void *addr, dynamic_handle_t handle,
-		char *fullPath, size_t fullPathLen);
+minpf_dynamic_library* minpf_load_dynamic_library(const char* path, char* error);
+bool minpf_unload_dynamic_library(minpf_dynamic_library* library);
+void* minpf_get_symbol(minpf_dynamic_library* library, const char* symbol);
+bool minpf_get_full_path(const char* path, void* addr, dynamic_handle_t handle, char* fullPath,
+						 size_t fullPathLen);
 
-}
+} // namespace grk
