@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -19,15 +19,14 @@
 
 #include "T1Interface.h"
 
-namespace grk {
-
-class T1Factory {
-public:
-	static T1Interface* get_t1(bool isCompressor,
-								TileCodingParams *tcp,
-								uint32_t maxCblkW,
-								uint32_t maxCblkH);
-
+namespace grk
+{
+class T1Factory
+{
+  public:
+	static T1Interface* makeT1(bool isCompressor, TileCodingParams* tcp, uint32_t maxCblkW,
+							   uint32_t maxCblkH);
+	static Quantizer* makeQuantizer(bool ht, bool reversible, uint8_t guardBits);
 };
 
-}
+} // namespace grk
