@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -17,15 +17,18 @@
 
 #pragma once
 
+namespace grk
+{
+struct CompressBlockExec;
+struct DecompressBlockExec;
 
-namespace grk {
+class T1Interface
+{
+  public:
+	virtual ~T1Interface() = default;
 
-class T1Interface {
-public:
-	virtual ~T1Interface() {}
-
-	virtual bool compress(CompressBlockExec *block)=0;
-	virtual bool decompress(DecompressBlockExec *block)=0;
+	virtual bool compress(CompressBlockExec* block) = 0;
+	virtual bool decompress(DecompressBlockExec* block) = 0;
 };
 
-}
+} // namespace grk
