@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -16,15 +16,16 @@
  */
 
 #pragma once
-namespace grk {
-
-class RateControl {
-public:
-	static void convexHull(CodePass *pass, uint32_t numPasses);
+namespace grk
+{
+class RateControl
+{
+  public:
+	static void convexHull(CodePass* pass, uint32_t numPasses);
 	static uint16_t slopeToLog(double slope);
 	static double slopeFromLog(uint16_t logSlope);
-private:
 
+  private:
 };
 
-}
+} // namespace grk
