@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -16,36 +16,34 @@
  */
 #include "grk_includes.h"
 
-namespace grk {
-
-RateInfo::RateInfo() :
-		minimumSlope(USHRT_MAX), maximumSlope(0) {
-
-}
-
+namespace grk
+{
+RateInfo::RateInfo() : minimumSlope(USHRT_MAX), maximumSlope(0) {}
 /*
  Synchronize with code block
  */
-void RateInfo::synch(CompressCodeblock *cblk) {
-	for (auto passno = 0U; passno < cblk->numPassesTotal; passno++) {
-		CodePass *pass = &cblk->passes[passno];
+void RateInfo::synch(CompressCodeblock* cblk)
+{
+	for(auto passno = 0U; passno < cblk->numPassesTotal; passno++)
+	{
+		CodePass* pass = &cblk->passes[passno];
 
-		//2. only process feasible truncation points
-		if (pass->slope == 0)
+		// 2. only process feasible truncation points
+		if(pass->slope == 0)
 			continue;
 
 		uint16_t s = pass->slope;
 
-		//4. update min and max
-		if (s < minimumSlope)
+		// 4. update min and max
+		if(s < minimumSlope)
 			minimumSlope = s;
-		if (s > maximumSlope)
+		if(s > maximumSlope)
 			maximumSlope = s;
 	}
 }
-
-uint16_t RateInfo::getMinimumThresh() {
+uint16_t RateInfo::getMinimumThresh()
+{
 	return minimumSlope;
 }
 
-}
+} // namespace grk
