@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -16,20 +16,19 @@
  */
 
 #pragma once
-namespace grk {
-
-class RateInfo {
-public:
+namespace grk
+{
+class RateInfo
+{
+  public:
 	RateInfo();
-	void synch(CompressCodeblock *cblk);
+	void synch(CompressCodeblock* cblk);
 
 	uint16_t getMinimumThresh();
 
-private:
-
+  private:
 	uint16_t minimumSlope;
 	uint16_t maximumSlope;
-
 };
 
-}
+} // namespace grk
