@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -19,19 +19,31 @@
 #pragma once
 #include <stdexcept>
 
-namespace grk {
-
-class DecodeUnknownMarkerAtEndOfTileException: public std::exception {};
-class PluginDecodeUnsupportedException: public std::exception {};
-class CorruptJP2BoxException: public std::exception {};
-class TruncatedPacketHeaderException: public std::exception {};
-class InvalidMarkerException: public std::exception {
-public:
-	explicit InvalidMarkerException(uint16_t marker) : m_marker(marker)
-	{}
-
-	uint16_t m_marker;
-
+namespace grk
+{
+class DecodeUnknownMarkerAtEndOfTileException : public std::exception
+{
 };
-class BadAsocException: public std::exception {};
-}
+class PluginDecodeUnsupportedException : public std::exception
+{
+};
+class CorruptJP2BoxException : public std::exception
+{
+};
+class TruncatedPacketHeaderException : public std::exception
+{
+};
+class CorruptPacketHeaderException : public std::exception
+{
+};
+class InvalidMarkerException : public std::exception
+{
+  public:
+	explicit InvalidMarkerException(uint16_t marker) : marker_(marker) {}
+
+	uint16_t marker_;
+};
+class BadAsocException : public std::exception
+{
+};
+} // namespace grk
