@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 2016-2021 Grok Image Compression Inc.
+ *    Copyright (C) 2016-2022 Grok Image Compression Inc.
  *
  *    This source code is free software: you can redistribute it and/or  modify
  *    it under the terms of the GNU Affero General Public License, version 3,
@@ -16,26 +16,23 @@
 
 #pragma once
 
-namespace grk {
-
-struct logger {
-
+namespace grk
+{
+struct logger
+{
 	logger();
-	void *m_error_data;
-	void *m_warning_data;
-	void *m_info_data;
+	void* error_data_;
+	void* warning_data_;
+	void* info_data_;
 	grk_msg_callback error_handler;
 	grk_msg_callback warning_handler;
 	grk_msg_callback info_handler;
 
-	static logger m_logger;
+	static logger logger_;
 };
 
+void GRK_INFO(const char* fmt, ...);
+void GRK_WARN(const char* fmt, ...);
+void GRK_ERROR(const char* fmt, ...);
 
-void GRK_INFO(const char *fmt,	...);
-void GRK_WARN(const char *fmt,	...);
-void GRK_ERROR(const char *fmt,...);
-
-
-
-}
+} // namespace grk
